from ppms import ppms
from . import info_handler
import time
import re

def identity(value):
    return value

def text_to_int(value):
    try:
        int_val = int(value)
    except Exception:
        return 0
    return int_val

def text_to_float(value):
    try:
        float_val = round(float(value), 3)
    except Exception:
        return 0.0
    return float_val

def date_to_msp_date(days_since_epoch, day_time):
    # the di value is the number of days since 1970. 
    # output should be somthing like this: 2005-08-24T08:00:00
    if days_since_epoch == 0:
        return None #no times given when this is the case
    date_time = time.gmtime((days_since_epoch * 3600 * 24) + (day_time * 60))
    return time.strftime('%Y-%m-%dT%H:%M:%S', date_time)    

def msp_date_to_planta_date(date_string):
    if date_string is None:
        return 0
    time_obj = time.strptime(date_string, '%Y-%m-%dT%H:%M:%S')
    seconds_since_epoch = time.mktime(time_obj)
    return int(seconds_since_epoch / 3600 / 24)
    
def start_date_to_msp_date(di_val):
    return date_to_msp_date(int(di_val), 8 * 60) #work day starts at 8:00

def finish_date_to_msp_date(di_val):
    return date_to_msp_date(int(di_val), 17 * 60) # work day finishs at 17:00

def day_begin_to_msp_date(di_val):
    return date_to_msp_date(int(di_val), 0) # calendar day begins at 00:00

def day_end_to_msp_date(di_val):
    return date_to_msp_date(int(di_val),(23 * 60) + 59) # calendar day ends at 23:59
    
def hours_to_duration_string(hours):
    # the output should look like "PT112H0M0S" for 112 hours 
    if hours < 0:
        # TODO: notify about that (WARN)
        hours = 0
    return 'PT{0}H0M0S'.format(int(hours))

def days_to_msp_hours(di_val):
    return hours_to_duration_string(int(di_val * 8))

def hours_to_msp_hours(di_val):
    return hours_to_duration_string(di_val)

def msp_hours_to_planta_hours(hour_string):
    if type(hour_string) != str:
        return 0
    hour_format = re.compile(r'^PT([0-9]*)H[0-9]*M[0-9]*S$')
    match_obj = hour_format.match(hour_string)
    return int(match_obj.groups()[0])

def msp_hours_to_planta_days(hour_string):
    hours = msp_hours_to_planta_hours(hour_string)
    if type(hours) == int:
        return int(hours / 8)
    else: return 0

def link_type_to_msp_link_type(di_val): #Values are 0=FF, 1=FS, 2=SF and 3=SS
    link_type_mapping = {
        1:'3', #PLANTA SS, MSP SS
        2:'2', #PLANTA SE, MSP SF
        3:'1', #PLANTA ES, MSP FS
        4:'0'  #PLANTA EE, MSP FF
    }
    
    if di_val in link_type_mapping:
        return link_type_mapping[di_val]
    else:
        return '1' #MSP FS

def msp_link_type_to_planta_link_type(value):
    link_type_mapping = {
        '0':4,  #PLANTA EE, MSP FF
        '1':3, #PLANTA ES, MSP FS
        '2':2, #PLANTA SE, MSP SF
        '3':1 #PLANTA SS, MSP SS
    }

    if value in link_type_mapping:
        return link_type_mapping[value]
    else:
        return 3

def link_lag_to_msp_link_lag(value):
    # value is float (in procentual case) or int (both number of days)
    # MSP has a quite funny unit for expressing the lag: 
    # "The amount of lag in tenths of a minute"
    return int(value * 8 * 60 * 10)

def msp_link_lag_to_planta_link_lag(value):
    try:
        return int(int(value) / 8 / 60 / 10)
    except TypeError:
        return 0
