from ppms import ppms
from . import information_box as info_box

LEVEL_INFO, LEVEL_WARN, LEVEL_ERROR = range(3)
summary = {LEVEL_INFO:0, LEVEL_WARN:0, LEVEL_ERROR:0}
level_names = {LEVEL_INFO:'Info', LEVEL_WARN:'Warn', LEVEL_ERROR:'ERROR'}

def _time_stamp():
    from time import strftime
    return strftime('%Y-%m-%dT%H:%M:%S')

def init_summary():
    for level in summary:
        summary[level] = 0

def notify_user(message, level):
    log_msg = '{0} - {1}: {2}'.format(_time_stamp(), level_names[level], message).encode()
    summary[level] += 1
    log_summary = 'Details ({0} Infos, {1} Warnings, {2} Errors)'.format(
        summary[LEVEL_INFO], summary[LEVEL_WARN], summary[LEVEL_ERROR])
    
    if level == LEVEL_INFO:
        info_box.show_info(message.encode())
    info_box.add_to_log(log_msg, log_summary)

def info(message):
    notify_user(message, LEVEL_INFO)

def warn(message):
    notify_user(message, LEVEL_WARN)
    
def error(message):
    notify_user(message, LEVEL_ERROR)

def silence_msgs():
    #clear the message stack
    while ppms.msg_pop() != None:
        pass
    ppms.msg_off()

def message_reply_to_text(reply):
    replies = {
        0:'ERROR',
        1:'OK/YES',
        2:'NO',
        3:'CANCEL'
    }
    return replies[reply]

def show_meantime_msgs():
    msg = ppms.msg_pop()
    msgs = list()
    
    while msg is not None:
        msgs.append(msg)
        msg = ppms.msg_pop()

    msgs.reverse()
    for msg in msgs:
        msg_text = 'Message catched: "{0}" was answered with "{1}"'.format(
            msg.get_text(), message_reply_to_text(msg.get_reply()))
        
        msg_id = msg.get_id()
        if msg_id in('0000', 'Python error', '0120'):
            error(msg_text)
        else:
            info(msg_text)

def re_verbose_msgs():
    ppms.msg_on()

def start_ui():
    info_box.start()
    silence_msgs()

def refocus():
    info_box.refocus()

def finish_ui():
    show_meantime_msgs()
    re_verbose_msgs()
    info('Finished, please have a look at the log.')
    info_box.finish()
