from io import StringIO
import sys

from . import transform_structure

from .. import info_handler
from .. import messages as msg

from ppms.file_transfer import put_file_on_client
from ppms import ppms

def export(project_id, export_filename=None):
    
    if not export_filename:
        export_filename = '{0}_export.xml'.format(project_id)
    
    export_memfile = StringIO()
    
    export_resources = False
    
    #prevent summing up when exporting multiple times in one session
    info_handler.init_summary()
    
    info_handler.start_ui()
        
    try:

        #Automatic PSP calculation
        schedule_mod = ppms.get_target_module()
        info_handler.info(msg.get_const('psp_generation'))
        schedule_mod.open_module('009AB7')

        root_obj = transform_structure.transform_project(project_id, export_resources)
        if root_obj is None:
            info_handler.info(msg.get_const('unsuccessful_export'))
            info_handler.finish_ui()
            return
        export_memfile.write('<?xml version="1.0" ?>\n')
        root_obj.export(export_memfile, 0)
        
        info_handler.info(msg.get_const('export_transfer'))
        put_file_on_client.save_file_dialog(title='Save export result in XML format',
            filter_string='XML Files (*.xml)|*.xml|All files (*.*)|*.*',
            restore_directory=True, proposed_filename=export_filename,
            bin_content=export_memfile.getvalue().encode())
        export_memfile.close()
    except Exception as err:
        info_handler.error(msg.get_const('export_exception') \
            + '\n%s' % repr(err))
        
    finally:
        info_handler.finish_ui()
