from . import transform_data
from . import default_values
from . import transform_calendar
from . import transform_task_structure as transform_task

from ..mspdi_2003_classes import *
from .. import format
from .. import fetch_project_data as fetch_data
from .. import info_handler
from .. import messages as msg
from .. import mspdi_extended_fields

from ppms import ppms

def get_boundaries(project_rec):
    calc_start = project_rec.calc_start.get_value()
    act_start = project_rec.actual_start.get_value()
    
    calc_end = project_rec.calc_end.get_value()
    act_end = project_rec.actual_end.get_value()

    proj_start = act_start if act_start > 0 and act_start < calc_start else\
        calc_start
    proj_end =  act_end if act_end > 0 and act_end > calc_end else calc_end

    return (proj_start, proj_end)
    
def transform_project(project_id, export_resources=True):
    
    info_handler.info(msg.get_const('project_export').format(project_id))

    #Project itself (root node in the exported XML)
    project_rec = fetch_data.get_proj_rec(project_id)
    project_obj = Project()
    
    #set default_values
    default_values.add_project_defaults(project_obj)

    #add PLANTA fields
    mspdi_extended_fields.set_planta_field_cust(project_obj)
    
    #set the exportable data from project record
    transform_data.transform_project_data(project_rec, project_obj)

    #Calendars
    boundaries = get_boundaries(project_rec)
    calendars = transform_calendar.transform_calendars(boundaries)
    if calendars is None:
        return
    project_obj.set_Calendars(calendars)

    if project_rec.pr_calendar.get_value() not in ('', '-'):
        pr_cal_uid = transform_calendar.get_extra_cal(
            project_rec.pr_calendar.get_value(), project_obj)
        project_obj.set_CalendarUID(pr_cal_uid)

    #Set actual project start (min(act_start, start))
    project_obj.set_StartDate(format.start_date_to_msp_date(boundaries[0]))

    #Tasks, do Resources and Assignments also there, so we need the Project instance 
    project_obj.set_Tasks(transform_task.transform_tasks(
        project_rec, project_obj, export_resources))
    
    return project_obj
