from ppms import ppms
from . import transform_data

from .. import info_handler
from .. import messages as msg
from .. import format
from .. import mspdi_extended_fields
from .. import task_hash
from ..mspdi_2003_classes import Task

#0: task in msp = task in planta (same by id and hash)
#1: task in msp != task in planta (same by id)
#2: task not in msp, but in planta
#3: task in msp, not in planta (has id)
#4: task in msp, not in planta (no id)
#5: same as MSP_DELETED, we would have to save the export first
#change indicators:
UNCHANGED, CHANGED, MSP_DELETED, PLANTA_DELETED, MSP_NEW, PLANTA_NEW = range(6)

def clear_additional_tasks(proj_rec):
    ppms.db_set_auto_commit(False)
    additional_rec_amount = ppms.db_modify("DELETE FROM MSPI_TASK WHERE PR_ID = "
            "'{0}' AND TASK_ID = '-'".format(proj_rec.pr_id.get_value()))
    info_handler.info(msg.get_const('unlinked_record_deletion').format(additional_rec_amount))
    ppms.db_commit()
    ppms.db_set_auto_commit(True)

def clear_task_data(task_rec):
    task_rec.duration.set_value(0)
    task_rec.start_date.set_value(0)
    task_rec.finish_date.set_value(0)
    task_rec.percent_complete.set_value(0.0)
    task_rec.task_name.set_value('')
    task_rec.milestone.set_value(0)
    task_rec.summary_task.set_value(0)
    task_rec.outline_code.set_value('')
    
def add_deleted_rec_reference(proj_rec, task_id):
    change_indicator = MSP_DELETED
    info_handler.info(msg.get_const('msp_deleted_task').format(task_id))
    msp_task = Task()
    ref_rec = transform_data.transform_task_comparison_data(proj_rec, msp_task, task_id, change_indicator)
    if ref_rec != None:
        if ref_rec.state() == ppms.DtpRecord.STORED:
            clear_task_data(ref_rec)
        #add change state indicator
        ref_rec.save()
    else:
        info_handler.warn(msg.get_const('task_reimport_failed').format(task_id))
    return ref_rec

def add_comparison_rec(proj_rec, msp_task, existing_tasks):
    planta_id = mspdi_extended_fields.get_extended_field_value(msp_task, 'TaskID')
    planta_hash = mspdi_extended_fields.get_extended_field_value(msp_task, 'TaskHash')
    planta_mspid = mspdi_extended_fields.get_extended_field_value(msp_task, 'MSPID')
    change_indicator = UNCHANGED

    if planta_id != None and planta_hash != None and planta_mspid == msp_task.get_UID():
        if planta_id in existing_tasks:
            if task_hash.calc_task_hash(msp_task) == planta_hash:
                info_handler.info(msg.get_const('unchanged_task').format(planta_id))
            else:
                info_handler.info(msg.get_const('changed_task').format(planta_id))
                change_indicator = CHANGED
            existing_tasks.remove(planta_id)
        else:
            info_handler.info(msg.get_const('planta_deleted_task').format(planta_id))
            planta_id = None
            change_indicator = PLANTA_DELETED
    else:
        info_handler.info(msg.get_const('msp_added_task').format(msp_task.get_UID()))
        change_indicator = MSP_NEW

    planta_id = '-' if planta_id is None or planta_mspid != msp_task.get_UID() else planta_id
    comparison_rec = transform_data.transform_task_comparison_data(proj_rec, msp_task, planta_id, change_indicator)
    if comparison_rec != None:
        #add change state indicator
        comparison_rec.save()
    else:
        info_handler.warn(msg.get_const('task_reimport_failed').format(str(msp_task.get_UID())))
    return comparison_rec

def clear_links(project_id):
    ppms.db_set_auto_commit(False)
    additional_rec_amount = ppms.db_modify("DELETE FROM MSPI_LINK WHERE EXISTS "
	 "(SELECT * FROM MSPI_TASK "
	 "WHERE MSPI_LINK.PRED_MSP_ID = MSPI_TASK.MSP_ID "
	 "AND MSPI_TASK.PR_ID = '{0}')".format(project_id))
    info_handler.info(msg.get_const('msp_link_table_cleaned').format(additional_rec_amount))
    ppms.db_commit()
    ppms.db_set_auto_commit(True)


def reimport_links(project_rec, task_ids):

    pr_id = project_rec.pr_id.get_value()
    clear_links(pr_id)
    
    for task_id in task_ids:
        succ_task, succ_rec = task_ids[task_id]
        for link in succ_task.get_PredecessorLink():
            pred_task, pred_rec = task_ids[link.get_PredecessorUID()]
            link_comp_rec = transform_data.transform_link_comparison_data(succ_rec, pred_rec, link)
            if link_comp_rec is not None:
                link_comp_rec.save()
                info_handler.info(msg.get_const('msp_link_added').format(
                    pred_rec.task_id.get_value(), succ_rec.task_id.get_value()))
            else:
                info_handler.warn(msg.get_const('msp_link_addition_failed').format(
                    pred_rec.task_id.get_value(), succ_rec.task_id.get_value()))

def reimport_tasks(project_instance, project_rec):
    task_ids = dict()
    task_outline_codes = dict()
    existing_tasks = [task_rec.task_id.get_value() for task_rec in project_rec.get_children(463)]

    clear_additional_tasks(project_rec)
    
    for task in project_instance.get_Tasks().get_Task():
        if task.get_UID() == '0' or task.get_IsNull() == '1':
            continue

        task_comparison_rec = add_comparison_rec(project_rec, task, existing_tasks)
        task_tuple = (task, task_comparison_rec)
        task_ids[task.get_UID()] = task_tuple
        task_outline_codes[task.get_OutlineNumber()] = task_tuple

    #add reference to mark tasks that where deleted in MSP
    for task_id in existing_tasks:
        task_comparison_rec = add_deleted_rec_reference(project_rec, task_id)
        #deleted_task_rec = ppms.search_record(463, [project_rec.pr_id.get_value(), task_id], , True)
        #task_outline_codes[

    #add further structure information: parent tasks based on OutlineNumber
    for code in task_outline_codes:
        if code.count('.') >= 1:
            task_rec = task_outline_codes[code][1]
            parent_rec = task_outline_codes[code[0:code.rfind('.')]][1]
            task_rec.parent_task.set_value(parent_rec.MSP_ID.get_value())
            task_rec.save()

    reimport_links(project_rec, task_ids)
    
