from ppms import ppms
from . import transform_data
from . import task_reimport
from .. import info_handler
from .. import messages as msg
from .. import format
from .. import mspdi_extended_fields as ext_fields
from .. import task_hash
from ..mspdi_2003_classes import Task

def build_task_structure(container, parent=None, prefix=''):
    import math
    if len(container) == 0:
        return
    largest_elem = max([int(elem) for elem in container.keys()])
    padding_len = int(math.log(largest_elem, 10)) + 1
    element_template = '{0:0>' + str(padding_len) + '}'
    for outline_num in container:
        element_code = prefix
        task, task_rec = container[outline_num][1]
        if parent is not None:
            element_code += '.'    
            task_rec.parent_task.set_value(parent.task_id.get_value())
            task_rec.parent_pr_id.set_value(parent.pr_id.get_value())
        element_code += element_template.format(outline_num)
        task_rec.vg_psp_code.set_value(element_code)
        task_rec.save()

        info_handler.info(msg.get_const('task_import').format(
            task_rec.task_id.get_value(), task.get_UID()))
        
        build_task_structure(container[outline_num][0], task_rec, element_code)

def get_structure_dict(task_outline_codes):
    outline_codes = sorted(task_outline_codes.keys(),
        key = lambda x: x.count('.'))
    structure = dict()
    for code in outline_codes:
        parts = code.split('.')
        container = structure
        for part in parts[0:-1]:
            container = container[part][0]
        container[parts[-1]] = (dict(), task_outline_codes[code])
    return structure

def add_new_task(proj_rec, task, fix_tasks):
    task_rec = transform_data.transform_task_data(proj_rec, task)
    try:
        if task_rec.requested_start.get_value() > task_rec.actual_start.get_value() > 0:
            task_rec.split.set_value(1)
        if fix_tasks:
            task_rec.fix.set_value(1)
    except AttributeError as e:
        info_handler.error(msg.get_const('invalid_task').format(
            str(task_rec.task_id.get_value()), e))
    return task_rec

def add_reimport_info(task, task_rec):
    ext_fields.add_extended_field(task, ('TaskID', task_rec.task_id.get_value()))
    ext_fields.add_extended_field(task, ('MSPID', task.get_UID()))
    task_hash.set_task_hash(task)

def transform_tasks(project_instance, project_rec, fix_tasks):
    #get all tasks
    task_outline_codes = dict()
    task_ids = dict()

    for task in project_instance.get_Tasks().get_Task():
        if task.get_UID() == '0' or task.get_IsNull() == '1':
            continue

        task_rec = add_new_task(project_rec, task, fix_tasks)
        
        # to be able to add the new project to the reimport table,
        # we have to fill out the PLANTA ID and the task hash in the 
        # task objects
        add_reimport_info(task, task_rec)
        
        task_tuple = (task, task_rec)
        task_outline_codes[task.get_OutlineNumber()] = task_tuple
        task_ids[task.get_UID()] = task_tuple

    build_task_structure(get_structure_dict(task_outline_codes))
    link_tasks(project_instance, project_rec, task_ids)
    
    #actually do the reimport step
    task_reimport.reimport_tasks(project_instance, project_rec)

def link_tasks(project_instance, project_rec, task_id_dict):
    for id in task_id_dict.keys():
        task, task_rec = task_id_dict[id]
        index = 0
        for link in task.get_PredecessorLink():
            pred_task, pred_rec = task_id_dict[link.get_PredecessorUID()]
            if pred_rec:
                index += 1
                #now link them using an AOB-rec
                aob = transform_data.transform_link_data(task_rec, pred_rec, link)
                if aob is not None:
                    aob.save()
                    info_handler.info(msg.get_const('task_linked').format(
                        pred_rec.task_id.get_value(), link.get_PredecessorUID(),
                        task_rec.task_id.get_value(), task.get_UID()))
                else:
                    info_handler.error(msg.get_const('task_link_failed').format(pred_rec.task_name.get_value(), task_rec.task_name.get_value()))
            else:
                info_handler.warn(msg.get_const('linked_task_missing').format(link.get_PredecessorUID(), task_rec.task_id.get_value()))
