from ppms import ppms
from ppms import ppms_cu
import time

import rpyc
from csrpc.utils import access

LEVEL_INFO, LEVEL_WARN, LEVEL_ERROR = range(3)
level_names = {LEVEL_INFO:'INFO', LEVEL_WARN:'WARN', LEVEL_ERROR:'ERROR'}

LOG_FMT = '{0} - {1}: {2}'
SUMMARY_FMT = 'Details ({0} Infos, {1} Warnings, {2} Errors)'
TIMESTAMP_FMT = '%Y-%m-%dT%H:%M:%S'
FINISH_TEXT = 'Finished, please have a look at the log.'
UIMSG_FMT = 'Message catched: "{0}" was answered with "{1}"'
EXCEPTION_FMT = "An unhandled exception occured: \n{0}."


def silence_msgs():
    #clear the message stack
    while ppms.msg_pop() != None:
        pass
    ppms.msg_off()


def message_reply_to_text(reply):
    replies = {
        0:'ERROR',
        1:'OK/YES',
        2:'NO',
        3:'CANCEL'
    }
    return replies[reply]


def re_verbose_msgs():
    ppms.msg_on()


def get_current_msgs():
    msg = ppms.msg_pop()
    messages = list()

    while msg is not None:
        message_text = UIMSG_FMT.format(msg.get_text(), 
            message_reply_to_text(msg.get_reply())
        )

        if msg.get_id() in ('0000', 'Python error', '0120'):
            messages.append((LEVEL_ERROR, message_text))
        else:
            messages.append((LEVEL_INFO, message_text))

        msg = ppms.msg_pop()

    return reversed(messages)


class InformationWindow(object):
    text_constants={
        "window_title": ppms_cu.Helper.get_const_title("001174"),
        "finish_button_title": ppms_cu.Helper.get_const_title("000032")
    }

    def __init__(self):
        self.info_window = access.remote_service('info_window')
        self.summary = {LEVEL_INFO:0, LEVEL_WARN:0, LEVEL_ERROR:0}

    #implement the context manager protocol:
    def __enter__(self):
        self.start_ui()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_val:
            self.error(EXCEPTION_FMT.format(str(exc_val)))
        self.finish_ui()
        return True  # no need to propagate an exception

    def start_ui(self):
        title = self.text_constants['window_title']
        fb_title = self.text_constants['finish_button_title']
        self.info_window.init_window(title=title, fb_title=fb_title)
        async_show = rpyc.async(self.info_window.show)
        async_show()
        silence_msgs()

    def notify_user(self, message, level):
        def timestamp():
            return time.strftime(TIMESTAMP_FMT)

        log_msg = LOG_FMT.format(timestamp(), level_names[level], message)
        self.summary[level] += 1
        log_summary = SUMMARY_FMT.format(self.summary[LEVEL_INFO],
            self.summary[LEVEL_WARN], self.summary[LEVEL_ERROR])

        if level == LEVEL_INFO:
            self.info_window.change_info(message)
        self.info_window.add_log_entry(log_msg)
        self.info_window.show_log_summary(log_summary)

    def info(self, message):
        self.notify_user(message, LEVEL_INFO)

    def warn(self, message):
        self.notify_user(message, LEVEL_WARN)

    def error(self, message):
        self.notify_user(message, LEVEL_ERROR)

    def refocus(self):
        self.info_window.refocus()

    def finish_ui(self):
        for level, text in get_current_msgs():
            self.notify_user(text, level)
        re_verbose_msgs()
        self.info(FINISH_TEXT)
        self.info_window.finish()

